#include "db.h"

#include "helpers\exception.h"

db::db(string const & name, int data_version)
:_obj(CLSID_QUICKDB), _obj_face(_obj)
{
	HRESULT hres=_obj_face->Init(name.c_str(), data_version);
	if (FAILED(hres))
		throw Exception(hres, "error");
}

db::~db()
{
}

/*************************
*     table creation      *
**************************/
void db::new_table()
{
	HRESULT hres=_obj_face->new_table();
	if (FAILED(hres))
		throw Exception(hres, "error");
}

int db::add_field(string const & field_name, EType field_type)
{
	int result=0;
	HRESULT hres=_obj_face->add_field(field_name.c_str(), field_type, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::add_field_ref(string const & field_name, string const & table_name)
{
	int result=0;
	HRESULT hres=_obj_face->add_field_ref(field_name.c_str(), table_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::commit_table(string const & table_name)
{
	int result=0;
	HRESULT hres=_obj_face->commit_table(table_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

/*************************
*     record creation     *
**************************/
void db::new_record(string const & table_name)
{
	HRESULT hres=_obj_face->new_record(table_name.c_str());
	if (FAILED(hres))
		throw Exception(hres, "error");
}

void db::new_record(int table_number)
{
	HRESULT hres=_obj_face->new_record(table_number);
	if (FAILED(hres))
		throw Exception(hres, "error");
}

//operations with strings
void db::set_field(string const & field_name, int field_value)
{
	HRESULT hres=_obj_face->set_field(field_name.c_str(), field_value);
	if (FAILED(hres))
		throw Exception(hres, "error");
}

void db::set_field(string const & field_name, long field_value)
{
	HRESULT hres=_obj_face->set_field(field_name.c_str(), field_value);
	if (FAILED(hres))
		throw Exception(hres, "error");
}

void db::set_field(string const & field_name, string field_value)
{
	HRESULT hres=_obj_face->set_field(field_name.c_str(), field_value.c_str());
	if (FAILED(hres))
		throw Exception(hres, "error");
}

void db::set_field_ref(string const & field_name, int field_value)
{
	HRESULT hres=_obj_face->set_field_ref(field_name.c_str(), field_value);
	if (FAILED(hres))
		throw Exception(hres, "error");
}

//operations with ints
void db::set_field(int field_number, long field_value)
{
	HRESULT hres=_obj_face->set_field(field_number, field_value);
	if (FAILED(hres))
		throw Exception(hres, "error");
}

void db::set_field(int field_number, int field_value)
{
	HRESULT hres=_obj_face->set_field(field_number, field_value);
	if (FAILED(hres))
		throw Exception(hres, "error");
}

void db::set_field(int field_number, string field_value)
{
	HRESULT hres=_obj_face->set_field(field_number, field_value.c_str());
	if (FAILED(hres))
		throw Exception(hres, "error");
}

void db::set_field_ref(int field_number, int field_value)
{
	HRESULT hres=_obj_face->set_field_ref(field_number, field_value);
	if (FAILED(hres))
		throw Exception(hres, "error");
}

int db::commit_record(int record_number)
{
	int result=0;
	HRESULT hres=_obj_face->commit_record(result, record_number);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

/***********************************
*     general record operations     *
************************************/
//operations with strings
bool db::record_exist(string const & table_name, int record_number)
{
	bool result=false;
	HRESULT hres=_obj_face->record_exist(table_name.c_str(), record_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}
		
int db::get_number_records(string const & table_name)
{
	int result=0;
	HRESULT hres=_obj_face->get_number_records(table_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

void db::delete_record(string const & table_name, int record_number)
{
	HRESULT hres=_obj_face->delete_record(table_name.c_str(), record_number);
	if (FAILED(hres))
		throw Exception(hres, "error");
}

//operations with ints
bool db::record_exist(int table_number, int record_number)
{
	bool result=false;
	HRESULT hres=_obj_face->record_exist(table_number, record_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::get_number_records(int table_number)
{
	int result=0;
	HRESULT hres=_obj_face->get_number_records(table_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

void db::delete_record(int table_number, int record_number)
{
	HRESULT hres=_obj_face->delete_record(table_number, record_number);
	if (FAILED(hres))
		throw Exception(hres, "error");
}

/***********************************
*     operations on definitions     *
************************************/
//operations with strings
EType db::get_field_type(string const & table_name, string const & field_name) 
{
	EType result=db_int;
	HRESULT hres=_obj_face->get_field_type(table_name.c_str(), field_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}
	
string db::get_field_ref_relation(string const & table_name, int field_number) 
{
	char const * result=0;
	HRESULT hres=_obj_face->get_field_ref_relation(table_name.c_str(), field_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::get_field_index(string const & table_name, string const & field_name) 
{
	int result=0;
	HRESULT hres=_obj_face->get_field_index(table_name.c_str(), field_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

string db::get_field_name(string const & table_name, int field_number) 
{
	char const * result=0;
	HRESULT hres=_obj_face->get_field_name(table_name.c_str(), field_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::get_number_field(string const & table_name)
{
	int result=0;
	HRESULT hres=_obj_face->get_number_field(table_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

//operations with ints
EType db::get_field_type(int table_number, string const & field_name) 
{
	EType result=db_int;
	HRESULT hres=_obj_face->get_field_type(table_number, field_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

string db::get_field_ref_relation(int table_number, int field_number) 
{
	char const * result=0;
	HRESULT hres=_obj_face->get_field_ref_relation(table_number, field_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::get_field_index(int table_number, string const & field_name) 
{
	int result=0;
	HRESULT hres=_obj_face->get_field_index(table_number, field_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

string db::get_field_name(int table_number, int field_number) 
{
	char const * result=0;
	HRESULT hres=_obj_face->get_field_name(table_number, field_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::get_number_field(int table_number) 
{
	int result=0;
	HRESULT hres=_obj_face->get_number_field(table_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

/*********************************
*     record value retrieval      *
**********************************/

//operations with strings
int db::get_field_int(string const & table_name, int record_number, string const & field_name)
{
	int result=0;
	HRESULT hres=_obj_face->get_field_int(table_name.c_str(), record_number, field_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

long db::get_field_long (string const & table_name, int record_number, string const & field_name)
{
	long result=0;
	HRESULT hres=_obj_face->get_field_long(table_name.c_str(), record_number, field_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

string db::get_field_string(string const & table_name, int record_number, string const & field_name)
{
	char const * result=0;
	HRESULT hres=_obj_face->get_field_string(table_name.c_str(), record_number, field_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::get_field_ref(string const & table_name, int record_number, string const & field_name)
{
	int result=0;
	HRESULT hres=_obj_face->get_field_ref(table_name.c_str(), record_number, field_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::get_number_children(string const & table_name, int record_number, string const & child_table_name)
{
	int result=0;
	HRESULT hres=_obj_face->get_number_children(table_name.c_str(), record_number, child_table_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::get_child(string const & table_name, int record_number, string const & child_table_name, int index_child)
{
	int result=0;
	HRESULT hres=_obj_face->get_child(table_name.c_str(), record_number, child_table_name.c_str(), index_child, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

//operations with ints
int db::get_field_int(int table_number, int record_number, int field_number)
{
	int result=0;
	HRESULT hres=_obj_face->get_field_int(table_number, record_number, field_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

long db::get_field_long(int table_number, int record_number, int field_number)
{
	long result=0;
	HRESULT hres=_obj_face->get_field_long(table_number, record_number, field_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

string db::get_field_string(int table_number, int record_number, int field_number)
{
	char const * result=0;
	HRESULT hres=_obj_face->get_field_string(table_number, record_number, field_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::get_field_ref(int table_number, int record_number, int field_number)
{
	int result=0;
	HRESULT hres=_obj_face->get_field_ref(table_number, record_number, field_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::get_number_children(int table_number, int record_number, int child_table)
{
	int result=0;
	HRESULT hres=_obj_face->get_number_children(table_number, record_number, child_table, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::get_child(int table_number, int record_number, int child_table_number, int index_child)
{
	int result=0;
	HRESULT hres=_obj_face->get_child(table_number, record_number, child_table_number, index_child, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

/**************************************
*     file operations                  *
***************************************/
bool db::read_from_file(string const & filename)
{
	bool result=false;
	HRESULT hres=_obj_face->read_from_file(filename.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

bool db::save_to_file(string const & filename)
{
	bool result=false;
	HRESULT hres=_obj_face->save_to_file(filename.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}
/**************************************
*     general database operations      *
***************************************/
void db::erase_data()
{
	HRESULT hres=_obj_face->erase_data();
	if (FAILED(hres))
		throw Exception(hres, "error");
}

bool db::table_exist(int table_number)
{
	bool result=false;
	HRESULT hres=_obj_face->table_exist(table_number, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

int db::get_table_index(string const & table_name)
{
	int result=0;
	HRESULT hres=_obj_face->get_table_index(table_name.c_str(), result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

string db::get_table_name(int table_index)
{
	char const * result=0;
	HRESULT hres=_obj_face->get_table_name(table_index, result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}

void db::delete_table(int table_index)
{
	HRESULT hres=_obj_face->delete_table(table_index);
	if (FAILED(hres))
		throw Exception(hres, "error");
}

int db::get_number_tables()
{
	int result=0;
	HRESULT hres=_obj_face->get_number_tables(result);
	if (FAILED(hres))
		throw Exception(hres, "error");
	return result;
}